"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const eventemitter3_1 = require("eventemitter3");
const request = require("request");
const Gateway_1 = require("./Gateway");
class Client extends eventemitter3_1.EventEmitter {
    constructor(config) {
        super();
        this.config = config;
        this.gateways = [];
        this.connected = false;
        this.ready = false;
        this.config = config;
        this.on("gatewayConfigured", () => {
            if (!this.ready && this.gateways.filter((x) => !x.configuration).length === 0) {
                this.ready = true;
                this.emit("ready");
            }
        });
    }
    get restGateway() {
        return this.gateways.find((x) => x.rest);
    }
    get restApp() {
        return this.restGateway.app.service("rest");
    }
    get messageService() {
        return this.restGateway.app.service("message");
    }
    get configuration() {
        return this.gateways.find((x) => x.configuration).configuration;
    }
    rest(method, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.restApp.get(method, { args });
        });
    }
    service(name, callback, first = true) {
        return __awaiter(this, void 0, void 0, function* () {
            const results = (yield Promise.all(this.gateways.map((x) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const result = yield callback(x.app.service(name));
                    result.__GATEWAY__ = x;
                    return result;
                }
                catch (e) {
                    return null;
                }
            })))).filter((x) => !!x);
            return first ? results[0] : results;
        });
    }
    get(name, id, first = true) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.service(name, (service) => __awaiter(this, void 0, void 0, function* () { return yield service.get(id); }), first);
        });
    }
    find(name, query, first = false) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.service(name, (service) => __awaiter(this, void 0, void 0, function* () { return yield service.find(query); }), first);
        });
    }
    addGauge(name, help, labelNames = []) {
        if (!this.connected) {
            throw new Error("Wait till after connecting to add gauges");
        }
        this.gateways.forEach((x) => x.addGauge(name, help, labelNames));
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.config.gateways.forEach((url) => __awaiter(this, void 0, void 0, function* () {
                    this.gateways.push(new Gateway_1.default(this, url));
                }));
                request({
                    uri: "https://gateway.discordservers.com/token",
                    method: "GET",
                    headers: {
                        Authorization: "Bearer " + process.env.AUTH_TOKEN,
                    },
                }, (err, resp, body) => {
                    if (err) {
                        return reject(err);
                    }
                    this.connected = true;
                    resolve();
                });
            });
        });
    }
}
exports.default = Client;
//# sourceMappingURL=Client.js.map