"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const moment = require("moment");
class AbstractCommand {
    constructor(client, context) {
        this.client = client;
        this.context = context;
    }
    reply(content) {
        return this.sendMessage(this.context.message.channel.id, content);
    }
    getMessage(message) {
        return new Promise((resolve, reject) => {
            this.client.messageService.get(message.id, { query: { channel: message.channel.id } }).then(resolve).catch(reject);
        });
    }
    deleteMessage(message) {
        return new Promise((resolve, reject) => {
            this.client.messageService.remove(message.id, { query: { channel: message.channel.id } }).then(resolve).catch(reject);
        });
    }
    editMessage(message, content) {
        return new Promise((resolve, reject) => {
            this.client.messageService.update(message.id, { content }, { query: { channel: message.channel.id } }).then(resolve).catch(reject);
        });
    }
    sendMessage(channel, content) {
        return new Promise((resolve, reject) => {
            this.client.messageService.create(content, { query: { channel: channel.id || channel } }).then(resolve).catch(reject);
        });
    }
    reactOk() {
        return new Promise((resolve, reject) => {
            this.client.rest("addMessageReaction", this.context.channel.id, this.context.message.id, "👍🏻")
                .then(resolve)
                .catch(reject);
        });
    }
    reactNotOk() {
        return new Promise((resolve, reject) => {
            this.client.rest("addMessageReaction", this.context.channel.id, this.context.message.id, "👎🏻")
                .then(resolve)
                .catch(reject);
        });
    }
    embedMessage(callback) {
        return new Promise((resolve, reject) => {
            const embed = {
                author: {
                    icon_url: this.client.configuration.user.avatarURL,
                    name: this.client.configuration.user.username,
                },
                color: 0x428BCA,
                fields: [],
                timestamp: moment().utc().toDate(),
            };
            callback(embed);
            this.reply({ embed }).then(resolve).catch(reject);
        });
    }
}
exports.default = AbstractCommand;
//# sourceMappingURL=AbstractCommand.js.map