"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const CommandContext_1 = require("./CommandContext");
class CommandHandler {
    constructor(client) {
        this.client = client;
        this.client.on("ready", () => {
            this.client.gateways.forEach((gateway) => {
                gateway.on("executeCommand", this.handleCommand.bind(this));
            });
        });
    }
    static matches(info, data) {
        return info.name === data.name;
    }
    handleCommand(message, commandData, ack) {
        return __awaiter(this, void 0, void 0, function* () {
            const cls = this.findCommand(commandData);
            if (!cls) {
                return;
            }
            const command = this.buildCommand(cls, message);
            const parameters = [];
            const metadata = Reflect.getMetadata("command", command);
            for (const key in metadata.parameters) {
                if (!metadata.parameters.hasOwnProperty(key)) {
                    continue;
                }
                const value = metadata.parameters[key];
                parameters.push(commandData.properties[key] || value.defaultValue);
            }
            yield command.process(...parameters);
            ack();
        });
    }
    buildCommand(cls, message) {
        return new cls(this.client, new CommandContext_1.default(message));
    }
    findCommand(commandData) {
        for (const data of Object.values(this.client.commands)) {
            if (CommandHandler.matches(data.info, commandData)) {
                return data.command;
            }
        }
        return undefined;
    }
}
exports.default = CommandHandler;
//# sourceMappingURL=CommandHandler.js.map