"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const Client_1 = require("./Client");
const CommandHandler_1 = require("./Command/CommandHandler");
const CommandInfo_1 = require("./Command/CommandInfo");
class CommandClient extends Client_1.default {
    constructor(config, commandDirectory) {
        super(config);
        this.commandDirectory = commandDirectory;
        this.commands = {};
        this.commandHandler = new CommandHandler_1.default(this);
    }
    connect() {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            yield this.loadCommands(this.commandDirectory);
            yield _super("connect").call(this);
            this.gateways.forEach((gateway) => {
                gateway.emit("registerCommands", Object.values(this.commands).map((x) => x.info));
            });
        });
    }
    loadCommands(directory) {
        return new Promise((resolve, reject) => {
            fs.readdir(directory, (err, files) => {
                if (err) {
                    return reject(err);
                }
                if (!files) {
                    return reject(new Error("No commands found"));
                }
                for (const file of files) {
                    if (path.extname(file) !== ".js" || file.indexOf("Abstract") >= 0) {
                        continue;
                    }
                    const cls = require(`${directory}/${file}`).default;
                    const command = new cls(this);
                    const info = new CommandInfo_1.default(cls);
                    this.commands[info.name] = { command, info };
                    resolve();
                }
            });
        });
    }
}
exports.default = CommandClient;
//# sourceMappingURL=CommandClient.js.map